(******************************************************************************
*                                  memTbObj                                   *
* here we define the basis of a memory object - like tableObj ...             *
******************************************************************************)
unit memTbObj;

interface

uses
   tblBase
   ,objects
   ,dbEngMem
   ;

type
   wtCollectionPtr = ^ wtCollection;
   wtCollection = object(TSortedCollection)

      generationMode : boolean;
      partialKeyNum  : byte; 
      { generated descendents should place the maimum key num here }

      constructor init(ALimit, ADelta : integer);
      function srchKey( key : pointer; 
                        var index : integer;
                        L, H : integer) : boolean;
      function srchPartialKey(key : pointer; keyPos : byte; 
                                { keyPos : 1 = only first field, 2 = first 2 .. }
                        var index : integer;
                        L, H : integer) : boolean;
      function srchFirstKey(key : pointer; 
                           var index : integer; 
                           cursor : integer) : boolean;
      function srchNextKey(key : pointer; 
                           var index : integer; 
                           cursor : integer) : boolean;
      function srchPrevKey(key : pointer;
                           var index : integer;
                           cursor : integer) : boolean;
      function srchFirstGeneration(key : pointer;
                                  var index : integer;
                                  cursor : integer) : boolean;
      function srchNextGeneration(key : pointer;
                                  var index : integer;
                                  cursor : integer) : boolean;

      (* descendents should have a compare that exists when partialKeyNum is
         found - to support srchPartialKey like operations .. *)

   end; { wtCollection object definition }

   memoryTableObjectPtr = ^ memoryTableObject;
   memoryTableObject = object(tableBaseObject)


     (* tbCollection : TSomethingCollection .. *)

     collectionCursor : integer; { points to the AT position of the cursor }

     (* current, 
        next,
        prev    : dataSOMETHINGPtr; { pointers to the data record the table is built of .. } 
      *)

      constructor init( name : string;
                        typ  : tbType;
                        strg : tbStorageTypes;
                        max  : longInt
                     );
      destructor done; virtual;
      procedure reset; { position at the start of the table .. } virtual;
      function eof : boolean; virtual; { true if at the last record .. }
      function whereInTable : longint; virtual;

   end; { memoryTableObject object definition }

   memIdxTablePtr = ^ memIdxTableObject;
   memIdxTableObject = object(memoryTableObject)

      procedure delIdxEntry(item : pointer); virtual; { needed to delete the idx entry }
      procedure addIdxEntry(item : pointer); virtual; { add an idx entry }

   end; { memIdxTableObject }

implementation

(******************************************************************************
*                           memoryTableObject.init                            *
******************************************************************************)
constructor memoryTableObject.init;
begin
   inherited init(new(databaseEngineMemoryPtr, init), name, typ, strg, max);
   (* descendents will initialize the TSOMETHINGCollection here 
   *)
end; {memoryTableObject.init}

(******************************************************************************
*                           memoryTableObject.reset                           *
******************************************************************************)
procedure memoryTableObject.reset;
begin
   inherited reset;
   collectionCursor := 0; { point to 0 ... }
   lastRead := -1; { indicate 0 }
end; {memoryTableObject.reset}

(******************************************************************************
*                            memoryTableObject.eof                            *
******************************************************************************)
function memoryTableObject.eof;
begin
(* descendents will perform :

   if (collectionCursor >= tbCollection.count) then
      eof := true
   else
      eof := false;

 *)
 eof := inherited eof;
end; {memoryTableObject.eof}

(******************************************************************************
*                           memoryTableObject.done                            *
******************************************************************************)
destructor memoryTableObject.done;
begin
end; {memoryTableObject.done}

(******************************************************************************
*                       memoryTableObject.whereInTable                        *
******************************************************************************)
function memoryTableObject.whereInTable;
begin
   if (lastRowFlag) then
      whereInTable := collectionCursor
   else
      whereInTable := collectionCursor - 1;
end; {memoryTableObject.whereInTable}

 { --- wtCollection .... --- }

(******************************************************************************
*                            wtCollection.srchKey                             *
* this is a binary table that searches in a range passed in l, h. Returns     *
* TRUE if found, false otherwise. If true index has the position ..           *
******************************************************************************)
function wtCollection.srchKey;
var
   i, c : integer;
begin
   srchKey := false; { l = low range, high = high range .. }
   while (l <= h) do begin
      i := (l + h) shr 1; { mid point .. binary search }
      if (generationMode) then
         c := (integer(items^[i]^) - genValueSearch)
      else
         c := compare(keyOf(items^[i]), key);
      if (c < 0) then
         l := i + 1
      else begin
         h := i - 1;
         if (c = 0) then begin
            srchKey := true; { found it .. }
            if (not duplicates) then
               l := i;
         end; { found a match .. }
      end; { not c < 0 }
   end; { while more to search for .. }
   index := l; { return what we found .. }
end; {wtCollection.srchKey}

(******************************************************************************
*                         wtCollection.srchPartialKey                         *
******************************************************************************)
function wtCollection.srchPartialKey;
var
   tempKeyNum : byte;
begin
   tempKeyNum := partialKeyNum;
   partialKeyNum := keyPos;
   srchPartialKey := srchKey(key, index, l, h);
   partialKeyNum := tempKeyNum;
end; {wtCollection.srchPartialKey}

(******************************************************************************
*                          wtCollection.srchFirstKey                          *
******************************************************************************)
function wtCollection.srchFirstKey;
begin
   srchFirstKey := srchNextKey(key, index, cursor - 1); 
   { srchnext will add 1 to cursor - 1, so we actually perform search first }
end; {wtCollection.srchFirstKey}

(******************************************************************************
*                          wtCollection.srchNextKey                           *
* returns the next match from the cursor position, if there is one.           *
******************************************************************************)
function wtCollection.srchNextKey;
var
   b : boolean;
   i : integer;
begin
   b := srchKey(key, index, cursor + 1, count - 1);
   srchNextKey := b;
   if (b) then repeat
      i := index;
      b := srchKey(key, index, cursor + 1, index - 1); 
      if (not b) then begin
         index := i; { did not find a match before index .. }
{         lastRead := i; }
      end;
   until (not b);
   (* the srchkey does a binary search, it is possible that the next
      row that matches is before the middle of the range, so we check 
      between the low range limit, to the found index - 1 *)
end; {wtCollection.srchNextKey}

(******************************************************************************
*                          wtCollection.srchPrevKey                           *
* returns the prev match from the cursor position, if there is one            *
******************************************************************************)
function wtCollection.srchPrevKey;
var
   b : boolean;
   i : integer;
begin
   b := srchKey(key, index, 0, cursor - 1);
   srchPrevKey := b;
   if (b) then repeat
      i := index;
      b := srchKey(key, index, index + 1, cursor - 1); 
      if (not b) then
         index := i; { did not find a match before index .. }
   until (not b);
   (* the srchkey does a binary search, it is possible that the prev
      row that matches is after the middle of the range, so we check 
      between the found index + 1 to the high limit *)
end; {wtCollection.srchPrevKey}

(******************************************************************************
*                              wtCollection.init                              *
******************************************************************************)
constructor wtCollection.init;
begin
   inherited init(ALimit, ADelta); { by default full comparison .. }
   partialKeyNum := 255; { maximum available }
   generationMode := false;
end; {wtCollection.init}

(******************************************************************************
*                       wtCollection.srchNextGeneration                       *
******************************************************************************)
function wtCollection.srchNextGeneration;
begin
   generationMode := true;
   srchNextGeneration := srchNextKey(key, index, cursor);
   generationMode := false;
end; {wtCollection.srchNextGeneration}

(******************************************************************************
*                      wtCollection.srchFirstGeneration                       *
******************************************************************************)
function wtCollection.srchFirstGeneration;
begin
   generationMode := true;
   srchFirstGeneration := srchFirstKey(key, index, cursor);
   generationMode := false;
end; {wtCollection.srchFirstGeneration}

(******************************************************************************
*                        memIdxTableObject.delIdxEntry                        *
******************************************************************************)
procedure memIdxTableObject.delIdxEntry;
begin
   { when the parent wants to delete an entry, it sets the entry from the
   parent's buffer, using the descendents "putDataToBuffer" method, and
   calls the descendents delIdxEntry to delete this thing .. }
end; { memIdxTableObject.delIdxEntry }

(******************************************************************************
*                        memIdxTableObject.addIdxEntry                        *
******************************************************************************)
procedure memIdxTableObject.addIdxEntry;
begin
end; { memIdxTableObject.addIdxEntry }

end.
